@@ -1,27 +1,26 @@
-from __future__ import print_function
 import json
 import logging
+from typing import Dict
 
+from itemadapter import is_item, ItemAdapter
 from w3lib.url import is_url
 
-from scrapy.commands import ScrapyCommand
+from scrapy.commands import BaseRunSpiderCommand
 from scrapy.http import Request
-from scrapy.item import BaseItem
 from scrapy.utils import display
-from scrapy.utils.conf import arglist_to_dict
 from scrapy.utils.spider import iterate_spider_output, spidercls_for_request
 from scrapy.exceptions import UsageError
 
+
 logger = logging.getLogger(__name__)
 
 
-class Command(ScrapyCommand):
-
+class Command(BaseRunSpiderCommand):
     requires_project = True
 
     spider = None
-    items = {}
-    requests = {}
+    items: Dict[int, list] = {}
+    requests: Dict[int, list] = {}
 
     first_response = None
 
@@ -32,31 +31,29 @@ class Command(ScrapyCommand):
         return "Parse URL (using its spider) and print the results"
 
     def add_options(self, parser):
-        ScrapyCommand.add_options(self, parser)
-        parser.add_option("--spider", dest="spider", default=None,
-            help="use this spider without looking for one")
-        parser.add_option("-a", dest="spargs", action="append", default=[], metavar="NAME=VALUE",
-            help="set spider argument (may be repeated)")
-        parser.add_option("--pipelines", action="store_true",
-            help="process items through pipelines")
-        parser.add_option("--nolinks", dest="nolinks", action="store_true",
-            help="don't show links to follow (extracted requests)")
-        parser.add_option("--noitems", dest="noitems", action="store_true",
-            help="don't show scraped items")
-        parser.add_option("--nocolour", dest="nocolour", action="store_true",
-            help="avoid using pygments to colorize the output")
-        parser.add_option("-r", "--rules", dest="rules", action="store_true",
-            help="use CrawlSpider rules to discover the callback")
-        parser.add_option("-c", "--callback", dest="callback",
-            help="use this callback for parsing, instead looking for a callback")
-        parser.add_option("-m", "--meta", dest="meta",
-            help="inject extra meta into the Request, it must be a valid raw json string")
-        parser.add_option("--cbkwargs", dest="cbkwargs",
-            help="inject extra callback kwargs into the Request, it must be a valid raw json string")
-        parser.add_option("-d", "--depth", dest="depth", type="int", default=1,
-            help="maximum depth for parsing requests [default: %default]")
-        parser.add_option("-v", "--verbose", dest="verbose", action="store_true",
-            help="print each depth level one by one")
+        BaseRunSpiderCommand.add_options(self, parser)
+        parser.add_argument("--spider", dest="spider", default=None,
+                            help="use this spider without looking for one")
+        parser.add_argument("--pipelines", action="store_true",
+                            help="process items through pipelines")
+        parser.add_argument("--nolinks", dest="nolinks", action="store_true",
+                            help="don't show links to follow (extracted requests)")
+        parser.add_argument("--noitems", dest="noitems", action="store_true",
+                            help="don't show scraped items")
+        parser.add_argument("--nocolour", dest="nocolour", action="store_true",
+                            help="avoid using pygments to colorize the output")
+        parser.add_argument("-r", "--rules", dest="rules", action="store_true",
+                            help="use CrawlSpider rules to discover the callback")
+        parser.add_argument("-c", "--callback", dest="callback",
+                            help="use this callback for parsing, instead looking for a callback")
+        parser.add_argument("-m", "--meta", dest="meta",
+                            help="inject extra meta into the Request, it must be a valid raw json string")
+        parser.add_argument("--cbkwargs", dest="cbkwargs",
+                            help="inject extra callback kwargs into the Request, it must be a valid raw json string")
+        parser.add_argument("-d", "--depth", dest="depth", type=int, default=1,
+                            help="maximum depth for parsing requests [default: %(default)s]")
+        parser.add_argument("-v", "--verbose", dest="verbose", action="store_true",
+                            help="print each depth level one by one")
 
     @property
     def max_level(self):
@@ -81,8 +78,8 @@ class Command(ScrapyCommand):
         else:
             items = self.items.get(lvl, [])
 
-        print("# Scraped Items ", "-"*60)
-        display.pprint([dict(x) for x in items], colorize=colour)
+        print("# Scraped Items ", "-" * 60)
+        display.pprint([ItemAdapter(x).asdict() for x in items], colorize=colour)
 
     def print_requests(self, lvl=None, colour=True):
         if lvl is None:
@@ -93,21 +90,21 @@ class Command(ScrapyCommand):
         else:
             requests = self.requests.get(lvl, [])
 
-        print("# Requests ", "-"*65)
+        print("# Requests ", "-" * 65)
         display.pprint(requests, colorize=colour)
 
     def print_results(self, opts):
         colour = not opts.nocolour
 
         if opts.verbose:
-            for level in range(1, self.max_level+1):
-                print('\n>>> DEPTH LEVEL: %s <<<' % level)
+            for level in range(1, self.max_level + 1):
+                print(f'\n>>> DEPTH LEVEL: {level} <<<')
                 if not opts.noitems:
                     self.print_items(level, colour)
                 if not opts.nolinks:
                     self.print_requests(level, colour)
         else:
-            print('\n>>> STATUS DEPTH LEVEL %s <<<' % self.max_level)
+            print(f'\n>>> STATUS DEPTH LEVEL {self.max_level} <<<')
             if not opts.noitems:
                 self.print_items(colour=colour)
             if not opts.nolinks:
@@ -118,7 +115,7 @@ class Command(ScrapyCommand):
         items, requests = [], []
 
         for x in iterate_spider_output(callback(response, **cb_kwargs)):
-            if isinstance(x, (BaseItem, dict)):
+            if is_item(x):
                 items.append(x)
             elif isinstance(x, Request):
                 requests.append(x)
@@ -147,9 +144,8 @@ class Command(ScrapyCommand):
             if not self.spidercls:
                 logger.error('Unable to find spider for: %(url)s', {'url': url})
 
-        # Request requires callback argument as callable or None, not string
-        request = Request(url, None)
-        _start_requests = lambda s: [self.prepare_request(s, request, opts)]
+        def _start_requests(spider):
+            yield self.prepare_request(spider, Request(url), opts)
         self.spidercls.start_requests = _start_requests
 
     def start_parsing(self, url, opts):
@@ -202,12 +198,15 @@ class Command(ScrapyCommand):
             self.add_items(depth, items)
             self.add_requests(depth, requests)
 
+            scraped_data = items if opts.output else []
             if depth < opts.depth:
                 for req in requests:
                     req.meta['_depth'] = depth + 1
                     req.meta['_callback'] = req.callback
                     req.callback = callback
-                return requests
+                scraped_data += requests
+
+            return scraped_data
 
         # update request meta if any extra meta was passed through the --meta/-m opts.
         if opts.meta:
@@ -223,18 +222,11 @@ class Command(ScrapyCommand):
         return request
 
     def process_options(self, args, opts):
-        ScrapyCommand.process_options(self, args, opts)
+        BaseRunSpiderCommand.process_options(self, args, opts)
 
-        self.process_spider_arguments(opts)
         self.process_request_meta(opts)
         self.process_request_cb_kwargs(opts)
 
-    def process_spider_arguments(self, opts):
-        try:
-            opts.spargs = arglist_to_dict(opts.spargs)
-        except ValueError:
-            raise UsageError("Invalid -a value, use -a NAME=VALUE", print_help=False)
-
     def process_request_meta(self, opts):
         if opts.meta:
             try:
